﻿<#
# MDT Deployment v2 - 20201205 - Stefan Peters - info@microcloud.nl
# 
# Disclaimer:
# - Usage of this script is at your own risk.
# - Check https://www.microcloud.nl for more information.
#
# Prerequisites
# - This script uses Hyper-V powershell modules, please install the Hyper-V role and required Features before using this script.
# - This script used Azure Powershell commandlets from the "Az" module. Please install this module: https://docs.microsoft.com/en-us/powershell/azure/install-az-ps?view=azps-5.1.0
#
#>

# MDT Parameters
$global:MDTRoot="D:\MDT\"                         # Add your MDT root here, and also add trailing backslash.
$global:MDTBootISO="D:\MDT\Boot\mdtboot_x64.iso"  # Put the full path of the MDT boot ISO in the variable.
$global:MDTComputernameFileName=$global:MDTRoot+"\Scripts\mdt_imageversion.txt" # This does not have to be changed.

# Hyper-V Parameters
$global:VMNamePrefix="MDT_"          # Prefix used for creating VM/VHD etc.
$global:VHDDiskSize = 127            # OS Disk size in GB, choose the size carefully.
$global:HyperVRootPath="E:\hyper-v\" # The root path for you Hyper-V folder, add trailing backslash.
$global:VMMemory=4GB                 # 4GB minimum recommended.
$global:VMCPUS=2                     # 2 minimum recommended.
$global:VMNetwork="NATSwitch"        # Enter the name of the Hyper-V switch that connects the VM to the MDT HOST and Internet

# Conversion Parameters
$global:tempconversionpath="f:\temp\"  # Path to a large enough TEMP folder for VHD conversion, also add trailing backslash.

# Azure Login Parameters - change to your needs
$global:azureupn = "username@tenantname.onmicrosoft.com"            # Add the UPN of the account that is allowed to upload to a storage account
$global:azuresecpwdfilename="c:\scripts\username.secpwd"            # Use a secure password file to store the password of the account, see "How to create a Secure Password file:" 
$global:azuresubscriptionid="111111111-2222-3333-4444-555555555555" # Add your Azure subscription ID in the variable.

# Azure Storage Account
$global:storageaccountRG  = "RG-WVDImageStorageAccount"    # Put the resourcesgroupname where the storage account is located in the Azure subscription.
$global:storageaccountURI = "https://wvdstorageaccount.blob.core.windows.net/wvdimages/" # Enter the path where to store the VHD files, make sure the path exists.

<#

How to create a Secure Password file:
1. Login to the server with the account that will run the script and use the secure password file. This does not have to be the same account as you are saving the password for.
2. Choose where to store the secpwd file. Enter that path in the $fn variable
3. Execute these 2 lines of powershell code. When prompted enter the password of the account. It will be stored secure in the username.secpwd file.

$fn="c:\scripts\username.secpwd"
Read-host -prompt "Enter the password to save in ($fn) " -assecurestring | convertfrom-securestring | out-file $fn

Security note: 
 - Anyone with the password of the account mentioned in "1" can reverse engineer the secpwd file back to the original password.

#>

Function AutoLogonAz
{  
  $LoggedONID = ""
  $LoggedONID = (Get-AzSubscription).SubscriptionId 
  If ($loggedONID -eq ($global:azuresubscriptionid))
  {
    Write-Host "Azure powershell already connected"
  } else
  {
    Write-Host "Azure powershell not connected, logging on."
    $azurePassword = cat $global:azuresecpwdfilename | ConvertTo-SecureString    
    $psCred = New-Object System.Management.Automation.PSCredential($global:azureupn, $azurePassword)
    Add-AzAccount -Credential $psCred
    Select-AzSubscription -SubscriptionId $global:azuresubscriptionid
  }  
} #/AutoLognAz


Function DeployMDT
{
  Write-Host -ForegroundColor Yellow "Deploy new MDT in a Box OS Deployment, hang on we need your input in a minute."  
  $timestamp  = (Get-Date).ToString("yyyyMMdd-HHmm") # This timestamp is the 'version' of the Image we are creating.
  $xLocalVHDSize=$($global:VHDDiskSize)*1024*1024*1024 # VHD size is in GB, need to convert it to bytes

  # Construct Hyper-V names and parameters
  $vmName=$($global:VMNamePrefix)+$timestamp
  $vmPath=$global:HyperVRootPath+$vmName
  $vmBootVHD=$vmpath+"\"+$vmname+".vhd"
  
  # Write VMName to TXT file in scripts folder. Add PowerShell script LTImdtimageversion.ps1 as item in tasksequence.
  "MDT_Imageversion - File created during start if this machines journey." | out-file $global:MDTComputernameFileName 
  " ImageVersion : $vmname " | out-file $global:MDTComputernameFileName -append
  
  # Create Hyper-V VM - Create Folder
  New-Item -Path $vmPath -ItemType Directory                  # Create the folder to store the Hyper-V machine.
  New-VHD -Path $vmBootVHD -Dynamic -SizeBytes $xLocalVHDSize # Create the VHD (dynamic), might need to convert to full later.
  
  # Create Hyper-V VM - Create VM
  New-VM -Name $vmName -MemoryStartupBytes ($global:VMMemory) -BootDevice CD -VHDPath $vmBootVHD -Path $vmPath -Generation 1 -Switch $($global:VMNetwork)
  Set-VMDvdDrive -VMName $vmName -ControllerNumber 1 -ControllerLocation 0 -Path $($global:MDTBootISO)
  Set-VMProcessor –VMName $vmName –Count ($global:VMCPUS)  
  Start-VM -Name $vmName
  Write-Host "Ready creating the Hyper-V machine and starting it, please connect to the console (not enhanced)"
  start-sleep 60
} #/DeployMDT


Function CleanVM 
{
  $vmlist=get-vm # List all Hyper-V VMs
  Write-Host
  Write-Host -foregroundcolor green "Overview of VMs:"
  If ($vmlist.count -lt 0)
    {
      Write-Host -ForegroundColor red "Sorry no VMs found."
    } else
    {
      $vmteller = 0
      Foreach ($vm in $vmlist)
        {
          Write-Host "  $($vmteller) - $($vm.name)"
          $vmteller++
        }
      $choice = read-host "Geef het nummer in van de VM"
      Write-Host -ForegroundColor Yellow "... Deleting VM: $($vmlist[$choice].name)"
      Write-Host -ForegroundColor Yellow "... Deleting VM: $($global:HyperVRootPath)$($vmlist[$choice].name)"      
      Do
      { 
        Write-Host -ForegroundColor red "Are you sure (Y/N) ? " -NoNewline
        $choice2 = read-host
      } While ($($choice2.ToLower()) -ne "y" -or $($choice2.ToLower()) -ne "y")
      Switch ($choice2)
        {
          'n' { exit }
          'y' {
                $vmlist[$choice] | Remove-VM -Force
                Remove-Item "$($global:HyperVRootPath)$($vmlist[$choice].name)" -Recurse
              }
        }
    }
} #/CleanVM


Function UploadVHD
{
  $vmlist=Get-VM # Get all VMs on local Hyper-V
  Write-Host
  Write-Host -foregroundcolor green "Hyper-V VM Overview:"
  If ($vmlist.count -lt 0)
    {
      Write-Host -ForegroundColor red "Sorry no VMs found."
    } else
    {
      $vmcounter = 0
      Foreach ($vm in $vmlist)
        {
          Write-Host "  $($vmcounter) - $($vm.name)"
          $vmcounter++
        }
      Write-Host
      $choice = read-host "Enter the number of your choice" 

      # Reading local VHD filename
      $localvhdname=(Get-VMHardDiskDrive -VMName $vmlist[$choice].Name).path
      $localvhdfilename = (Split-Path $localvhdname -leaf).ToLower()
      
      # Construct URL to upload     
      $urlOfUploadedVhd = $global:storageaccountURI+$localvhdfilename
      
      # Construct temporary conversion filename
      $localfixedvhdfilename=$global:tempconversionpath+$localvhdfilename
      
      # Start conversion from dynamic VHD to fixed VHD     
      $startconverstiontimestamp=(Get-Date).ToString("yyyyMMdd HH:mm:ss") # Read start time.
      Write-Host -ForegroundColor Green "Start conversion from dynamic vhd to fixed vhd, can take some time, started at: $($startconverstiontimestamp)" 
      Convert-VHD -path $localvhdname -destinationpath $localfixedvhdfilename -vhdtype fixed
      $stopconverstiontimestamp=(Get-Date).ToString("yyyyMMdd HH:mm:ss")
      Write-Host -ForegroundColor Green "Conversion finished at: $($stopconverstiontimestamp)" 
      # End converstion

      # Start upload to Azure
      $startuploadtimestamp=(Get-Date).ToString("yyyyMMdd HH:mm:ss")
      Write-Host -ForegroundColor Yellow "Start uploading $($localfixedvhdfilename), this can take some time, started at: $($startuploadtimestamp)"        

      # JIT logon to Azure.
      AutoLogonAz # connect to azure
      Add-AzVhd -ResourceGroupName $global:storageaccountRG -Destination $urlOfUploadedVhd -LocalFilePath $localfixedvhdfilename # Upload the files      
      $stopuploadtimestamp=(Get-Date).ToString("yyyyMMdd HH:mm:ss")
      Write-Host -ForegroundColor Yellow "Upload complete $($localfixedvhdfilename), please check for errors... timestamp: $($stopuploadtimestamp)"      
      #
  }

} #/UploadVHD


Function MainMenu
{
  CLS
  Write-Host -ForegroundColor green "MDT in a Box - MainMenu"
  Write-Host
  Write-Host "A - Deploy new MDT OSD" 
  Write-Host "B - Convert and Upload a VHD to Azure" 
  Write-Host "C - Clean Hyper-V VM"
  Write-Host
  $choice=(Read-host "Choose (A-C)").ToLower()
  Switch ($choice)
  {
    'a' { DeployMDT }
    'b' { UploadVHD }    
    'c' { CleanVM }
  }
}

MainMenu